@@ -7,9 +7,15 @@
 #include "sieve.h"
 #include "constants.h"   /* _MPU_FILL_EXTRA_N and _MPU_INITIAL_CACHE_SIZE */
 
-#include "EXTERN.h"
-#include "perl.h"
-#include "XSUB.h"
+#ifdef STANDALONE
+  #undef USE_ITHREADS
+  #define MUTEX_INIT(x)
+  #define MUTEX_LOCK(x)
+  #define MUTEX_UNLOCK(x)
+  #define MUTEX_DESTROY(x)
+  #define COND_INIT(x)
+  #define COND_DESTROY(x)
+#endif
 
 /*
  * These functions are used internally by the .c and .xs files.
@@ -158,10 +164,12 @@ UV get_prime_cache(UV n, const unsigned char** sieve)
 #endif
 }
 
+#ifdef USE_ITHREADS
 void release_prime_cache(const unsigned char* mem) {
   (void)mem; /* We don't currently care about the pointer */
   READ_LOCK_END;
 }
+#endif
 
 
 
@@ -170,7 +178,7 @@ void release_prime_cache(const unsigned char* mem) {
 static unsigned char* prime_segment = 0;
 static int prime_segment_is_available = 1;
 /* If that's in use, malloc a new one of this size */
-#define SECONDARY_SEGMENT_CHUNK_SIZE  UVCONST( 64*1024-16)
+#define SECONDARY_SEGMENT_CHUNK_SIZE  UVCONST(128*1024-16)
 
 unsigned char* get_prime_segment(UV *size) {
   unsigned char* mem;
