@@ -13,6 +13,8 @@
 
 /* if n is smaller than this, you can multiply without overflow */
 #define HALF_WORD (UVCONST(1) << (BITS_PER_WORD/2))
+/* This will be true if we think mulmods are fast */
+#define MULMODS_ARE_FAST 1
 
 #if (BITS_PER_WORD == 32) && HAVE_STD_U64
 
@@ -59,29 +61,26 @@
   }
   #define addmod(a,b,n) _addmod(a,b,n)
 
-#elif BITS_PER_WORD == 64 && __GNUC__ == 4 && __GNUC_MINOR__ >= 4 && (defined(__x86_64__) || defined(__powerpc64__))
+#elif BITS_PER_WORD == 64 && defined(HAVE_UINT128)
 
   /* We're 64-bit, using a modern gcc, and the target has some 128-bit type.
    * The actual number of targets that have this implemented are limited. */
 
-  #if __GNUC__ == 4 && __GNUC_MINOR__ >= 4 && __GNUC_MINOR__ < 6
-    typedef unsigned int uint128_t __attribute__ ((__mode__ (TI)));
-  #else
-    typedef unsigned __int128 uint128_t;
-  #endif
-
   #define mulmod(a,b,n) (UV)( ((uint128_t)(a) * (b)) % (n) )
   #define sqrmod(a,n)   (UV)( ((uint128_t)(a) * (a)) % (n) )
 
 #else
 
   /* UV is the largest integral type available (that we know of). */
+#undef MULMODS_ARE_FAST
+#define MULMODS_ARE_FAST 0
 
   /* Do it by hand */
   static INLINE UV _mulmod(UV a, UV b, UV n) {
     UV r = 0;
     if (a >= n) a %= n;   /* Careful attention from the caller should make */
     if (b >= n) b %= n;   /* these unnecessary.                            */
+    if ((a|b) < HALF_WORD) return (a*b) % n;
     if (a < b) { UV t = a; a = b; b = t; }
     if (n <= (UV_MAX>>1)) {
       while (b > 0) {
@@ -99,20 +98,20 @@
     return r;
   }
 
-  #define mulmod(a,b,n) ((((a)|(b)) < HALF_WORD) ? ((a)*(b))%(n):_mulmod(a,b,n))
-  #define sqrmod(a,n)   (((a) < HALF_WORD)       ? ((a)*(a))%(n):_mulmod(a,a,n))
+  #define mulmod(a,b,n) _mulmod(a,b,n)
+  #define sqrmod(a,n)   _mulmod(a,a,n)
 
 #endif
 
 #ifndef addmod
-  #define addmod(a,b,n) ((((n)-(a)) > (b))  ?  ((a)+(b))  :  ((a)+(b)-(n)))
+  static INLINE UV addmod(UV a, UV b, UV n) {
+    return ((n-a) > b) ?  a+b  :  a+b-n;
+  }
 #endif
 
-/* We need to make sure a and b get evaluated into UVs, then do the
- * subtract into a UV before the addmod. */
 static INLINE UV submod(UV a, UV b, UV n) {
-  UV t1 = n - b;
-  return addmod(a, t1, n);
+  UV t = n-b;  /* Evaluate as UV, then hand to addmod */
+  return addmod(a, t, n);
 }
 
 /* a^2 + c mod n */
