@@ -19,6 +19,8 @@ extern int _XS_is_frobenius_underwood_pseudoprime(UV n);
 extern int _XS_is_almost_extra_strong_lucas_pseudoprime(UV n, UV increment);
 extern int is_mersenne_prime(UV p);
 extern int lucas_lehmer(UV p);
+extern int lucasu(IV* U, IV P, IV Q, UV k);
+extern int lucasv(IV* V, IV P, IV Q, UV k);
 
 extern int _XS_BPSW(UV const n);
 extern int is_prob_prime(UV n);
